#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

# Version neu: 14.02.2005


# Fehlerkorrektur an: 1, aus: 0
ERROR_CORRECTION=0


from math import sqrt
from re import sub
from string import upper,find

def mittelwert(seq):
   # Mittelwert eines Arrays zurckgeben
   m=0.0
   for i in seq:
      m = m+i
   return m/len(seq)

def abweichung(seq):
   # Standardabweichung eines Arrays zurckgeben
   mw = mittelwert(seq)
   n = len(seq)
   sum = 0
   for v in seq:
      sum = sum + (v-mw)**2
   return sqrt ( sum / (n-1) )
   
def median(seq):
   # Median-Wert eines Arrays zurckgeben
   seq2 = []
   for v in seq: seq2.append(v)
   seq2.sort()
   return seq2 [ len(seq2)/2 ]

def medsplit(seq,med):
   s1 = []  # Werte <= med
   s2 = []  # Werte > med
   for v in seq:
      if v <= med:
         s1.append(v)
      else:
         s2.append(v)
   return (s1,s2)

def fixvals(A,B):
   # Aufrumen: Werte, die nher am Mittelwert der anderen Folge sind,
   # rberschieben
   a = mittelwert(A)
   b = mittelwert(B)
   A2B = []   # Werte, die von A nach B wandern sollen
   B2A = []   # Werte, die von B nach A wandern sollen

   for v in A+B:
      if v in A and abs(v-a) > abs (v-b): A2B.append(v)
      if v in B and abs(v-b) > abs (v-a): B2A.append(v)

   for v in A2B:
      A.remove(v)
      B.append(v)
   for v in B2A:
      B.remove(v)
      A.append(v)

   if max(A) > min(B): print "Fehler in fixvals()"

   return (A,B)



codewords = [
    "0000000000", "0000011010", "0000101001", "0000110011",
    "0001000111", "0001011101", "0001101110", "0001110100",
    "0010000110", "0010011100", "0010101111", "0010110101",
    "0011000001", "0011011011", "0011101000", "0011110010",
    "0100000101", "0100011111", "0100101100", "0100110110",
    "0101000010", "0101011000", "0101101011", "0101110001",
    "0110000011", "0110011001", "0110101010", "0110110000",
    "0111000100", "0111011110", "0111101101", "0111110111",
    "1000000011", "1000011001", "1000101010", "1000110000",
    "1001000100", "1001011110", "1001101101", "1001110111",
    "1010000101", "1010011111", "1010101100", "1010110110",
    "1011000010", "1011011000", "1011101011", "1011110001",
    "1100000110", "1100011100", "1100101111", "1100110101",
    "1101000001", "1101011011", "1101101000", "1101110010",
    "1110000000", "1110011010", "1110101001", "1110110011",
    "1111000111", "1111011101", "1111101110", "1111110100"]


### distance()
### Hamming-Distanz zwischen zwei Worten
def distance(x,y):
    dist = 0
    for i in range(0,len(x)):
        if x[i] != y[i]:  dist+=1
    return dist
### end distance()


### cc_error_decode_one
### Diese Funktion wandelt einen fehlerkorrektur-kodierten
### 10-Bit-Block in einen 6-Bit-Block zurck.
def cc_error_decode_one (x):
    result = ""
    dist=50000  # groe Lnge als Anfangswert
    for w in codewords:
        if distance (x, w) < dist:
            result = w
            dist = distance (x, w)
    return result[0:6]
### end cc_error_decode_one


### cc_error_decode
### Diese Funktion wandelt fehlerkorrektur-kodierte 10-Bit-Blcke
### wieder in 6-Bit-Blcke um.
def cc_error_decoder (s,index):
    if ERROR_CORRECTION == 0:
        result = s       #  Fehlerkorrektur deaktiviert
    else:
        result = s[:index]
        t = s[index:]    #  Bits am Anfang ignorieren (unvollstaendig)
        for i in range (0,len(t)/10):
            partword = t[10*i:10*i+10]
            result += cc_error_decode_one(partword)
    return result
### end cc_error_decode


### cc_bits_to_ascii
### Diese Funktion wandelt einen 01-String wieder in ASCII zurueck.
### als zweites Argument wird die Anzahl der am Anfang zu ignorierenden
### Bits ausgegeben.
def cc_bits_to_ascii (s,index):
   result = ""
   pos = 0
   v = 0
   if index != 0: result=result+"?"
   for c in s[index:]:
      if c == "1": v = v+2**(5-pos)
      pos = pos+1
      if pos == 6:
         pos = 0
         result = result + chr (v+32)   # Zeichen anh-bngen-A
         v = 0
   return result
### end cc_bits_to_ascii


### cc_ascii_to_bits
### Diese Funktion wandelt eine ASCII-Nachricht in eine Folge
### aus Nullen und Einsen. Es werden nur Zeichen zwischen ASCII 32 und 95
### kodiert (= 64 Zeichen = 6 Bit). Startsequenz: 32, 95
# modifizierte Version: ohne Header und Footer
def cc_ascii_to_bits (s):
   result = ""  # ohne (!) Start-Sequenz
   s = upper(s)
   for c in s:
      if c < chr(32) or c > chr (95): c = chr(32)
      v = ord(c)-32        # i jetzt zwischen 0 und 63
      for i in [5,4,3,2,1,0]:
         if v & (2**i):
            result=result+"1"
         else:
            result=result+"0"
   return result
### end cc_ascii_to_bits


### bitfehler
### Vergleich zweier {0,1}-Wrter; Rckgabe der Anzahl der Abweichungen
def bitfehler (s1,s2):
   length = min (len(s1),len(s2))
   s1 = s1[:length]  # abschneiden
   s2 = s2[:length]
   errcount = []
   for i in range(0,length):
      if s1[i] != s2[i]: errcount.append(i)
   return errcount
### end bitfehler


def over005s(s):
   cnt = 0
   for v in s:
      if v > 0.05: cnt += 1
   return cnt

infile=open("data.in","r")
#outfile=open("data.out","w")

s=infile.read()
infile.close()

vals=[]
for v in s.split():
   vals.append(float(v))

med = median(vals)
mw = mittelwert(vals)
(vals1,vals2) = medsplit(vals,med)

print "Gesamtwerte"
print "Median:     %1.10f" % med
print "Mittelwert: %1.10f" % mw
print "Abweichung: %1.10f" % abweichung(vals)

print "Unterer Block"
print "Minimum:    %1.10f" % min(vals1)
print "Maximum:    %1.10f" % max(vals1)
print "Mittelwert: %1.10f" % mittelwert(vals1)
print "Abweichung: %1.10f" % abweichung(vals1)

print "Oberer Block"
print "Minimum:    %1.10f" % min(vals2)
print "Maximum:    %1.10f" % max(vals2)
print "Mittelwert: %1.10f" % mittelwert(vals2)
print "Abweichung: %1.10f" % abweichung(vals2)

(vals1,vals2) = fixvals(vals1,vals2)

print "AUFRAEUMEN"
print "Unterer Block"
print "Minimum:    %1.10f" % min(vals1)
print "Maximum:    %1.10f" % max(vals1)
print "Mittelwert: %1.10f" % mittelwert(vals1)
print "Abweichung: %1.10f" % abweichung(vals1)

print "Oberer Block"
print "Minimum:    %1.10f" % min(vals2)
print "Maximum:    %1.10f" % max(vals2)
print "Mittelwert: %1.10f" % mittelwert(vals2)
print "Abweichung: %1.10f" % abweichung(vals2)

# trenner = (min(vals2)-max(vals1))/2
trenner = max(vals1)+0.0001
# trenner = 0.1

msgbin = ""
for v in vals:
   if v < trenner:
      msgbin = msgbin+"0"
   else:
      msgbin = msgbin+"1"

print "Nachricht: ",msgbin


# Fehlerkorrektur anwenden, falls Variable gesetzt ist.
# Dadurch werden die 10-Bit-Blcke wieder zu 6-Bit-Blcken.
if ERROR_CORRECTION == 1:
   msgbin = cc_error_decoder (msgbin, 0)


# index=find(msgbin,"11111000000")
# msgbin=msgbin[index+11:]
msgbin=msgbin[12:]
print "Nachricht: ",msgbin

#  fhrenden Mll lschen

print "Text:      ", cc_bits_to_ascii(msgbin,0)

originaltext = """0@ .,NN'.:9I1]D4<S';]&S>ASUL,K?L&,KO5'\S0<TOY"W5SGR6]D*;P+9&I@&A02 =UA*VH>!]R%M.GP8Q)P@..ZO-SMU:$HT/Z7/!E$ *Y%.3HM^2#**5QS"*21(N _"""

print "Original:  ", originaltext

bf = bitfehler (msgbin,cc_ascii_to_bits(originaltext))
print "Bitfehler: ", len(bf),": ",bf
print "Laenge:    ", len(cc_ascii_to_bits(originaltext))
print "Fehler %:  ", (len(bf)+0.0)/(len(cc_ascii_to_bits(originaltext))+0.0)*100

print "Anzahl > 0,05s:", over005s (vals)
